package org.act.services.connector;
import java.util.ArrayList;
import java.util.List;
import org.act.metaset.Metaset;
import org.act.services.ConfigException;
import org.act.services.IMessageContext;
import org.act.services.ITransportContext;
import org.act.services.pool.IObjectPool;
import org.act.services.protocol.MessageProtocol;
import org.act.services.runtime.IRuntime;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.ObjectProducerThreadPoolRunnable;
import org.act.services.thread.Terminatable;
import org.act.services.thread.ThreadPool;

public class MessageConnector extends AObjectPoolConnector {
	
	private ThreadPool threadPool;
	private IObjectPool transportInputPool;
	private IObjectPool transportoutputPool;
	private IObjectPool msgInputPool;
	private IObjectPool msgOutputPool;
	private Terminatable trans2msg;
	private Terminatable msg2trans;
	
	private List messages = new ArrayList();;
	
	
	public MessageConnector(IRuntime runtime) {
		super(runtime);
	}
	public void stop() {
		trans2msg.terminate();
		synchronized (transportInputPool) {
			transportInputPool.notifyAll();
		}
		
		msg2trans.terminate();
		synchronized (msgOutputPool) {
			msgOutputPool.notifyAll();
		}
	}
	public void start() {
		Thread t = new Thread(trans2msg);		t.setName("Message In");
		t.start();
		Thread t2 = new Thread(msg2trans);		t2.setName("Message Out");
		t2.start();
	}
	
	public void config(Metaset conf) throws ConfigException {
		super.config(conf);
		String trans = conf.selector(".\\","inputpool");
		String msg = conf.selector(".\\","outputpool");
		msgInputPool = runtime.getObjectPool(msg, IObjectPool.IN_MODE);
		msgOutputPool = runtime.getObjectPool(msg, IObjectPool.OUT_MODE);
		transportInputPool = runtime.getObjectPool(trans, IObjectPool.IN_MODE);
		transportoutputPool = runtime.getObjectPool(trans, IObjectPool.OUT_MODE);
		
				String tpName = conf.selector(".\\threadpool","name");		threadPool = runtime.getThreadPool(tpName);		
				Metaset protocols = conf.filter(".\\protocol");
		for (int i=0; i<protocols.getChilds().size(); i++) {
			try {
				Metaset protocol = (Metaset) protocols.getChilds().get(i);
				String classname = protocol.selector(".\\","class");
				Class cls = Class.forName(classname);
				if (MessageProtocol.class.isAssignableFrom(cls)) {
					MessageProtocol messageProtocol = (MessageProtocol) cls.newInstance();
					messageProtocol.config(protocol);
					messages.add(messageProtocol);
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ConfigException e) {
				e.printStackTrace();
			}
		}
						trans2msg = new Terminatable() {
			
			protected void process() throws ObjectOperationException {
								ITransportContext request = null;
				synchronized (transportInputPool) {
					while (transportInputPool.empty()) {
						switch (getState()) {
						case RUNNING:
							try {
								transportInputPool.wait();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							break;
						case STOPPING:
							return;
						default:
							throw new IllegalMonitorStateException();
						}
					}
					request = (ITransportContext) transportInputPool.get();
					transportInputPool.notifyAll();
					}
							for (int i=0; i<messages.size(); i++) {
				MessageProtocol comsumer = (MessageProtocol) messages.get(i);
				Object adaptee = comsumer.adapt(request);
				if (adaptee!=null && adaptee instanceof ObjectProducerThreadPoolRunnable) {
					ObjectProducerThreadPoolRunnable runnable = (ObjectProducerThreadPoolRunnable)adaptee;
					runnable.setReceiver(msgInputPool);
															threadPool.runIt(runnable);					break;
				}
			}
			}
		};
		
		msg2trans = new Terminatable() {
			
			protected void process() throws ObjectOperationException {
								IMessageContext request = null;
				synchronized (msgOutputPool) {
					while (msgOutputPool.empty()) {
						switch (getState()) {
						case RUNNING:
							try {
								msgOutputPool.wait();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							break;
						case STOPPING:
							return;
						default:
							throw new IllegalMonitorStateException();
						}
					}
					request = (IMessageContext) msgOutputPool.get();
					msgOutputPool.notifyAll();
				}
								for (int i=0; i<messages.size(); i++) {
					MessageProtocol comsumer = (MessageProtocol) messages.get(i);
					Object adaptee = comsumer.adapt(request);
					if (adaptee!=null && adaptee instanceof ObjectProducerThreadPoolRunnable) {
						ObjectProducerThreadPoolRunnable runnable = (ObjectProducerThreadPoolRunnable)adaptee;
						runnable.setReceiver(transportoutputPool);
												threadPool.runIt(runnable);
						break;
					}
				}
			}
		};
	}
	public Object adapt(Object obj) {
		return null;
	}
}
