package org.act.services.protocol.soap;
import java.util.ArrayList;
import org.act.metaset.Metaset;
import org.act.services.ConfigException;
import org.act.services.IMessageContext;
import org.act.services.ITransportContext;
import org.act.services.MessageContext;
import org.act.services.pool.IObjectPool;
import org.act.services.protocol.MessageProtocol;
import org.act.services.protocol.http.HTTPTransportContext;
import org.act.services.runtime.Application;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.ObjectProducerThreadPoolRunnable;
import org.act.services.wsrm.P2PRMP;
import org.act.services.wsrm.ReliableMessage;

public class SOAP12MessageProtocol extends MessageProtocol {
	
	public SOAP12MessageProtocol() {
		super();
	}
	public Object adapt(Object obj) {
		if (obj instanceof ITransportContext) {
			return new SoapRequestProcessor((ITransportContext) obj);
		}
		else if (obj instanceof IMessageContext) {
			return new SoapResponseProcessor((IMessageContext) obj);
		}
		return null;
	}
	public void config(Metaset config) throws ConfigException {
	}
	public void start() {
	}
	public void stop() {
	}
}

class SoapRequestProcessor implements ObjectProducerThreadPoolRunnable {
	private ITransportContext request;
	private P2PRMP rmp = P2PRMP.getInstance();
	
	public SoapRequestProcessor(ITransportContext request) {
		this.request = request;
	}
	
	public IMessageContext createMessageContext(ITransportContext obj)
			throws ObjectOperationException {
		HTTPTransportContext transport = (HTTPTransportContext) obj;
		IMessageContext messageContext = new MessageContext();
		messageContext.setServicesName(transport.getPath());
				try {
						Metaset request = transport.getRequestTransportContext().getContent();
						verifyRequestMessage(request);
			messageContext.setRequestServices(request);
			messageContext.setRequestAttachments(transport.getRequestTransportContext().getAttachments());
			messageContext.setResponseTransportContext(transport
					.getResponseTransportContext());
		} catch (Exception ex) {
			ex.printStackTrace();
		}
				return messageContext;
	}
	
	private boolean verifyRequestMessage(Metaset request) {
						return true;
	}
	private IObjectPool messageInputPool;
	public void setReceiver(IObjectPool receiver) {
	this.messageInputPool = receiver;
	}
	public Object[] getInitData() {
		return null;
	}
	public void runIt(Object[] thData) {
		IMessageContext response = null;
		try {
			response = createMessageContext(request);
						if (Application.isDebug()) {
				System.out.println("Message Layer(Input):");
				System.out.println(response.getRequestServices().toDocument());
			}
		} catch (ObjectOperationException e1) {
			e1.printStackTrace();
		}
		if (response == null) {
			return;
		}
		ArrayList AL = null;
		AL = sendToRMP(response, AL);	
		putRmToMessageRequestPool(AL);
		
			}
	public void putRmToMessageRequestPool(ArrayList AL) {
	if ( ( AL == null ) || ( AL.size() <= 0 ) ) {
		return;
	}
	for ( int i = 0; i < AL.size() ; i ++ ) {
		ReliableMessage rm = (ReliableMessage)AL.get(i);
		synchronized (messageInputPool) {
			while (messageInputPool.full()) {
				try {
					messageInputPool.wait();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			messageInputPool.put( rm.getIMessageContext() );
			messageInputPool.notifyAll();
		}
	}
	}
	public ArrayList sendToRMP(IMessageContext response, ArrayList AL) {
	long BeginTime = System.currentTimeMillis();
	AL = rmp.SoapIN(response);
	long EndTime = System.currentTimeMillis();
	if (Application.isDebug()) {
	System.out.println("RMP Time ( Request ): " + (EndTime - BeginTime) + "milliseconds");
	}
	return AL;
	}
}

class SoapResponseProcessor implements ObjectProducerThreadPoolRunnable {
	private IMessageContext messageContext;
	private P2PRMP rmp = P2PRMP.getInstance();
	
	public SoapResponseProcessor(IMessageContext messageContext) {
		this.messageContext = messageContext;
	}
	private IObjectPool transportOutputPool;
	public void setReceiver(IObjectPool receiver) {
	this.transportOutputPool = receiver;
	}
	public Object[] getInitData() {
		return null;
	}
	public void runIt(Object[] thData) {
		ArrayList AL = null;
				AL = receiveFromRMP(messageContext, AL);	
				putRmTransportContextToTransportResponsePool(AL);
	}
	
	public ArrayList receiveFromRMP(IMessageContext messageContext, ArrayList AL) {
	long BeginTime = System.currentTimeMillis();
		AL = rmp.SoapOUT(messageContext);
	long EndTime = System.currentTimeMillis();
	if (Application.isDebug()) {
	System.out.println("RMP Time ( Response ): " + (EndTime - BeginTime) + "milliseconds");
	}
	return AL;
	}
	
	public void putRmTransportContextToTransportResponsePool(ArrayList AL) {
	if ( AL.size() <= 0 ) {
		return;
	}
	synchronized (transportOutputPool) {
		while (transportOutputPool.full()) {
			try {
				transportOutputPool.wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		ReliableMessage rm;
		for ( int i = 0; i < AL.size() ; i ++ ) {
			rm = (ReliableMessage)AL.get(i);
						ITransportContext.ResponseTransportContext rtc = 
				rm.getIMessageContext().getResponseTransportContext();
			rtc.setContent(rm.getIMessageContext().getResponseServices());
			rtc.setAttachments(rm.getIMessageContext().getResponseAttachments());
			if (Application.isDebug()) {
				System.out.println("Message Layer(Output):");
				System.out.println(rm.getIMessageContext().getResponseServices().toDocument());
			}
			transportOutputPool.put(rtc);
			transportOutputPool.notifyAll();
		}
	}
	}
}
