
package org.act.services.transaction.manager;


import org.apache.log4j.*;
import java.util.Vector;
import java.util.Hashtable;
import org.act.services.transaction.exception.TxException;
import org.act.services.transaction.information.*;
import org.act.services.transaction.messages.*;
import org.act.services.transaction.participant.*;
import org.act.services.transaction.util.*;
public abstract class Superior extends Actor implements Resource{
	
    protected TxControl control = null;
    
    public String txId = null;
   
    
    protected Superior(Category logger)
    {
        super(logger);
        control = TxControl.newInstance(logger);
    }
    
    public Message onEnrol(Enrol enrol)
    {
        String infId = enrol.getInferiorIdentifier();
                        boolean responseRequested = enrol.getResponseRequested();
        Actor from = this;
        InferiorResource ir = new InferiorResource(enrol, from);
        control.registerInferior(infId, ir);
        DBLog.logInferior(ir);
        logger().debug("[" + infId + "]Ǽ");
                if (responseRequested)
        {
            Enrolled enrolled = new Enrolled();
            enrolled.setInferiorIdentifier(infId);
            enrolled.setTransactionId(txId);
            return enrolled;
        }
        else
        {
            return reportFault("responseRequested parameter value of"
                               + " enrol message must be true");
        }
    }
    
    public Message onResign(Resign resign)
    {
        String infId = resign.getInferiorIdentifier();
        control.deRegisterInferior(infId);
        DBLog.logInferiorStatus(infId, Status.STATUS_RESIGNED);
        logger().debug("[" + infId + "]Ǽ");
        Resigned resigned = new Resigned();
        resigned.setInferiorIdentifier(infId);
        resigned.setTransactionId(txId);
        return resigned;
    }
    
    protected Message onRequestStatus(RequestStatus msg)
    {
                String status = Status.STATUS_UNKNOWN;
        status = control.getTxStatus();
        Status statusMsg = new Status(status);
        statusMsg.setTransactionId(txId);
        return statusMsg;
    }
    
    protected Message onRequestInferiorStatuses(RequestInferiorStatuses msg)
    {
        try
        {
            Vector inferiors = msg.getInferiorsList();
            String superiorId = msg.getTargetIdentifier();
            logger().debug("ʼѯSuperior[" + superiorId + "]Ĳߵ״̬");
            InferiorStatuses inferiorSatuses = new InferiorStatuses(superiorId); ;
            if (inferiors != null)
            {
                for (int i = 0; i < inferiors.size(); i++)
                {
                    String infId = (String)inferiors.get(i);
                    String status = control.getInferiorStatus(infId);
                    inferiorSatuses.addStatusItem(infId, status);
                }
            }
            logger().debug("ѯSuperior[" + superiorId + "]Ĳߵ״̬");
            inferiorSatuses.setTransactionId(txId);
            return inferiorSatuses;
        }
        catch (Exception e)
        {
            return reportFault("error in request inferior statuses :"
                               + e.getMessage());
        }
    }
    
    protected Message onPrepareInferiors(PrepareInferiors msg)
    {
        try
        {
            String superiorId = TxUri.buildUri(getWebServiceUrl(), txId);
            logger().debug("ʼprepare superior[" + superiorId + "]ָInferior");
            Vector inferiors = msg.getInferiorIdentifiers();
            Hashtable result = control.prepareInferiors(txId, inferiors);
            InferiorStatuses replyMsg = new InferiorStatuses(superiorId);
            java.util.Iterator it = result.entrySet().iterator();
            while (it.hasNext())
            {
                java.util.Map.Entry en = (java.util.Map.Entry)it.next();
                String infId = (String)en.getKey();
                String status = (String)en.getValue();
                replyMsg.addStatusItem(infId, status);
            }
            logger().debug("prepare superior[" + superiorId +
                           "]ָInferior,InferiorStatuses");
            replyMsg.setTransactionId(txId);
            return replyMsg;
        }
        catch (TxException e)
        {
            return reportFault("prepare inferiors error:" + e.getMessage());
        }
    }
    
    protected Message onCancelInferiors(CancelInferiors msg)
    {
        try
        {
            String superiorId = TxUri.buildUri(getWebServiceUrl(), txId);
            Vector inferiors = msg.getInferiorIdentifiers();
            logger().debug("ʼcancel superior[" + superiorId + "]ָInferior");
            Hashtable result = control.cancelInferiors(txId, inferiors);
            InferiorStatuses replyMsg = new InferiorStatuses(superiorId);
            java.util.Iterator it = result.entrySet().iterator();
            while (it.hasNext())
            {
                java.util.Map.Entry en = (java.util.Map.Entry)it.next();
                String infId = (String)en.getKey();
                String status = (String)en.getValue();
                replyMsg.addStatusItem(infId, status);
            }
            logger().debug("cancel superior[" + superiorId +
                           "]ָInferior,InferiorStatuses");
            replyMsg.setTransactionId(txId);
            return replyMsg;
        }
        catch (TxException e)
        {
            return reportFault("cancel inferiors error:" + e.getMessage());
        }
    }
    
    protected Message reportFault(String text)
    {
        Fault fault = new Fault();
        fault.setFaultType(Fault.FAULT_General);
        fault.setFaultText(text);
        fault.setTransactionId(txId);
        return fault;
    }
    
    protected Message reportUnknownTransaction(String txid)
    {
        Fault fault = new Fault();
        fault.setFaultType(Fault.FAULT_UnkownTransaction);
        fault.setFaultText("Transaction [" + txid + "] does not exist");
        fault.setTransactionId(txId);
        return fault;
    }
        protected Message onRedirect(Redirect msg)
    {
        return reportFault("superior should not receive Redirect message");
    }
    protected Message onContextReply(ContextReply msg)
    {
        return reportFault("superior should not receive ContextReply message");
    }
    protected Message onConfirmed(Confirmed msg)
    {
        return reportFault("superior should not receive Confirmed message");
    }
  
    protected Message onCancelled(Cancelled msg)
    {
        return reportFault("superior should not receive Cancelled message");
    }
    protected Message onStatus(Status msg)
    {
        return reportFault("superior should not receive Status message");
    }
    protected Message onApplicationMessage(ApplicationMessage appMsg)
    {
        return reportFault(
            "superior should not receive ApplicationMessage message");
    }
    
    protected Message onFault(Fault msg)
    {
        return reportFault("superior should not receive Fault message");
    }
    
    protected Message onHazard(Hazard msg)
    {
        String infId = msg.getInferiorIdentifier();
        Contradiction contradiction = new Contradiction();
        contradiction.setInferiorIdentifier(infId);
        contradiction.setTransactionId(txId);
        return contradiction;
    }
    
    public TxControl getTxControl()
    {
        return this.control;
    }
    
    public String getTransactionId()
    {
        return this.txId;
    }
    
    public void setTransactionId(String txId_)
    {
        this.txId = txId_;
    }
        public void enrolled(String transactionid)
    {}
    public int prepare(String transactionid)
        throws TxException
    {
        return Vote.VOTE_PREPARED;
    }
    public void confirm(String transactionid)
        throws TxException
    {}
    public void confirm_one_phase(String transactionid)
        throws TxException
    {}
    public void cancel(String transactionid)
        throws TxException
    {}
}
