
package org.act.services.transaction.manager;


import java.util.*;
import org.act.services.transaction.information.*;
import org.act.services.transaction.messages.Status;
import org.act.services.transaction.participant.*;
import org.act.services.transaction.exception.TxException;
import org.act.services.transaction.util.TxUri;
import org.apache.log4j.Category;
public class TxControl {
	      public static final int TX_CONFIRMED = 0;
    public static final int TX_CANCELLED = 1;
    public static final int TX_FAULT = 2;
    
    private SuperiorContext sc = null;
    
    private Category logger = null;
        
    private Object MsgObj=new Object();
    private String ObjId=null;
    
        private long prepareTime=0;
    private long confirmTime=0;
    
    private TxControl(Category log)
    {
        this.logger = log;
    }
    
    public static TxControl newInstance(Category log)
    {
        return new TxControl(log);
    }
    
    public void setSuperiorContext(SuperiorContext sc)
    {
        this.sc = sc;
    }
    
    public SuperiorContext getSuperiorContext()
    {
        return sc;
    }
    
    public void setConfirmSet(Vector confirmSet)
    {
        if (sc != null)
        {
            sc.setConfirmSet(confirmSet);
        }
    }
    
    public Vector getConfirmSet()
    {
        if (sc != null)
        {
            return sc.getConfirmSet();
        }
        return null;
    }
    
    public void registerInferior(String inferiorId,
                                 InferiorResource res)
    {
        if (sc != null)
        {
            sc.setInferiorResource(inferiorId, res);
        }
    }
    
    public void deRegisterInferior(String inferiorId)
    {
        if (sc != null)
        {
            sc.removeInferiorReource(inferiorId);
        }
    }
    
    public void setTxStatus(String status)
    {
        if (sc != null)
        {
                        if (isTopTransaction())
            {
                RunningData.setTxStatus(sc.getTransactionId(), status);
            }
                        sc.setTxStatus(status);
                        DBLog.logSuperiorStatus(sc);
        }
    }
    
    public String getTxStatus()
    {
        if (sc != null)
        {
            return sc.getTxStatus();
        }
        return TxStatus.TX_UNKNOWN;
    }
    
    public String getInferiorStatus(String infId)
    {
        String status = Status.STATUS_UNKNOWN;
        if (sc != null)
        {
            InferiorResource ir = sc.getInferiorResource(infId);
            status = ir.getSatus();
        }
        return status;
    }
    
    public boolean getTxType()
    {
        boolean atom = true;
        if (sc != null)
        {
            atom = sc.isAtom();
        }
        return atom;
    }
    
    public boolean isTopTransaction()
    {
        boolean top = true;
        if (sc != null)
        {
            top = sc.isTopTransaction();
        }
        return top;
    }
    
    public int prepare(String txId)
        throws TxException
    {
                          if (sc == null)
         {
             setTxStatus(TxStatus.TX_FAULT);
             System.err.println("Transaction:" + txId + " does not exist");
             throw new TxException(null,"Transaction:" + txId + " does not exist",null,null,null);
         }
        String txStatus = getTxStatus();
        String txType = sc.isAtom() ? "ԭ" : "";
                if (txStatus == TxStatus.TX_CONFIRMED ||
            txStatus == TxStatus.TX_CANCELLED)
        {
            logger.debug(txType + "" + txId + "Ѿ" + txStatus + ", Ҫٴύ");
            setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null,"Transaction" + txId + " has already " +
                                  txStatus,null,null,null);
        }
                if (txStatus == TxStatus.TX_PREPARED)
        {
            return Vote.VOTE_PREPARED;
        }
        if (txStatus == TxStatus.TX_ABORTED)
        {
            return Vote.VOTE_ABORTED;
        }
                 int ret = Vote.VOTE_PREPARED;
        boolean atom = sc.isAtom();
        String errMsg = null;
        boolean isPrepared = true;
        setTxStatus(TxStatus.TX_PREPARING);
        logger.debug(txType + "" + txId + "ʼPREPARE");
                if (atom)
        {
            Vector infs = sc.getInferiorList();
            for (int i = 0; i < infs.size(); i++)
            {
                String infId = (String)infs.get(i);
                InferiorResource res = (InferiorResource)sc.getInferiorResource(
                    infId);
                System.err.println("*****************"+res.getInferiorId()+"prepare*******************");
                                MsgObj=res;
                ObjId=res.getInferiorId();
                                int status = res.prepare(txId);
                switch (status)
                {
                    case Vote.VOTE_RESIGN:
                        sc.removeInferiorReource(infId);
                        break;
                    case Vote.VOTE_PREPARED:
                        break;
                    case Vote.VOTE_ABORTED:
                        isPrepared = false;
                        break;
                    case Vote.VOTE_FAULT:
                        ret = Vote.VOTE_FAULT;
                        errMsg = errMsg + "\n Inferior[" +
                                 TxUri.getUrlFromUri(infId) +
                                 "]cannot prepare";
                        break;
                    default:
                }
            }
        }
                 if (!atom)
         {
             Vector confirmSet = sc.getConfirmSet();
             if (confirmSet == null)
             {
                 confirmSet = sc.getInferiorList();
             }
             for (int i = 0; i < confirmSet.size(); i++)
             {
                 String infId = (String)confirmSet.get(i);
                 InferiorResource res = (InferiorResource)sc.
                                        getInferiorResource(infId);
                 if (res == null)
                 {
                     logger.debug("[" + infId + "],Ѿ˳");
                     continue;
                 }
                 String status = res.getSatus();
                 if (status == Status.STATUS_PREPARED)
                 {
                     logger.debug("[" + infId + "]Ҫprepare,Ѿ" + status);
                     continue;
                 }
                 if (status == Status.STATUS_CANCELLED ||
                     status == Status.STATUS_CONFIRMED ||
                     status == Status.STATUS_ABORTED)
                 {
                     logger.debug("[" + infId + "]prepare,Ѿ" + status);
                     errMsg = errMsg + "\n Inferior(in confirmset)[" +
                              TxUri.getUrlFromUri(infId) +
                              "]can not prepare because already " + status;
                     continue;
                 }
                 int vote = res.prepare(txId);
                 switch (vote)
                 {
                     case Vote.VOTE_RESIGN:
                         sc.removeInferiorReource(infId);
                         break;
                     case Vote.VOTE_PREPARED:
                         break;
                     case Vote.VOTE_ABORTED:
                         isPrepared = false;
                         break;
                     case Vote.VOTE_FAULT:
                         ret = Vote.VOTE_FAULT;
                         errMsg = errMsg + "\n Inferior(in confirmset)[" +
                                  TxUri.getUrlFromUri(infId) +
                                  "]can not prepare";
                         break;
                     default:
                 }
             }
         }
                          setTxStatus(TxStatus.TX_PREPARED);
        if (errMsg != null)
        {
            setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null, errMsg, null, null, null);
        }
                else if (isPrepared == false)
        {
            setTxStatus(TxStatus.TX_ABORTED);
            ret = Vote.VOTE_ABORTED;
        }
                if (sc.getInferiorCount() == 0)
        {
            setTxStatus(TxStatus.TX_REDIGNED);
            ret = Vote.VOTE_RESIGN;
        }
        return ret;
    }
    
    public int confirmOnly(String txId)
        throws TxException
    {
                 int result = TxControl.TX_FAULT;
        String error = null;
        if (sc == null)
        {
                        setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null,"Transaction" + txId + " does not exist",null,null,null);
        }
        else         {
            String status = sc.getTxStatus();
            String txType = sc.isAtom() ? "ԭ" : "";
            if (status == Status.STATUS_CANCELLED
                || status == Status.STATUS_CONFIRMED
                || status == Status.STATUS_ABORTED)
            {
                logger.debug(txType + "" + txId + "Ѿ" + status);
                setTxStatus(TxStatus.TX_FAULT);
                throw new TxException(null,"Transaction:" + txId + " has already " +
                                      status,null,null,null);
            }
        }
                 boolean atom = sc.isAtom();
        String txType = sc.isAtom() ? "ԭ" : "";
        setTxStatus(TxStatus.TX_CONFIRMING);
        logger.debug(txType + "" + txId + "ʼCONFIRM");
                if (atom)
        {
            Vector infs = sc.getInferiorList();
            for (int i = 0; i < infs.size(); i++)
            {
                String infId = (String)infs.get(i);
                InferiorResource res = (InferiorResource)sc.getInferiorResource(
                    infId);           
                                MsgObj=res;
                ObjId=res.getInferiorId();
                                boolean status = res.confirm(txId);
                if (!status)
                {
                    error = error + "\n inferior[" + TxUri.getUrlFromUri(infId) +
                            "]cannot commit";
                }
                                sc.setInferiorSatuts(infId,res.getSatus());
            }
        }
                 if (!atom)
         {
                          Vector confirmInfs = sc.getConfirmSet();
             if (confirmInfs == null)
             {
                 confirmInfs = sc.getInferiorList();
             }
             for (int i = 0; i < confirmInfs.size(); i++)
             {
                 String infId = (String)confirmInfs.get(i);
                 InferiorResource res = (InferiorResource)
                                        sc.getInferiorResource(infId);
                 if (res == null)
                 {
                     logger.debug("[" + infId + "],Ѿ˳");
                     continue;
                 }
                 String status = res.getSatus();
                 if (status == Status.STATUS_CONFIRMED)
                 {
                     logger.debug("[" + infId + "]Ѿconfirmed");
                     continue;
                 }
                 if (status == Status.STATUS_CANCELLED ||
                     status == Status.STATUS_ABORTED)
                 {
                     logger.debug("[" + infId + "]Ѿ" + status +
                                  ",ܽconfirm");
                     error = error + "\n  inferior(in confirmset)[" +
                             TxUri.getUrlFromUri(infId)
                             + "] has  cancelled before " + "\n";
                     continue;
                 }
                                  boolean confirmResult = res.confirm(txId);
                 if (!confirmResult)
                 {
                     error = error + "\n inferior(in confirmset)[" +
                             TxUri.getUrlFromUri(infId)
                             + "]cannot confirm" + "\n";
                 }
             }
                                       Vector otherInfs = sc.getNoConfirmSet();
             for (int i = 0; i < otherInfs.size(); i++)
             {
                 String infId = (String)otherInfs.get(i);
                 InferiorResource res = (InferiorResource)sc.
                                        getInferiorResource(infId);
                 String status = res.getSatus();
                 if (status != Status.STATUS_CANCELLED)
                 {
                     boolean cancelResult = res.cancel(txId);
                     if (!cancelResult)
                     {
                                                                       }
                 }
             }
         }
                 if (error == null)
         {
             result = TxControl.TX_CONFIRMED;
             setTxStatus(TxStatus.TX_CONFIRMED);
                                   }
         else
         {
             setTxStatus(TxStatus.TX_FAULT);
             throw new TxException(null,error,null,null,null);
         }
        return result;
    }
    
    public int confirm(String txId)
        throws TxException
    {
                if (sc == null)
        {
            throw new TxException(null,"Transaction:" + txId + " does not exist",null,null,null);
        }
        String txStatus = sc.getTxStatus();
        String txType = sc.isAtom() ? "ԭ" : "";
                if (txStatus == TxStatus.TX_CONFIRMED ||
            txStatus == TxStatus.TX_CANCELLED)
        {
            logger.debug(txType + "" + txId + "Ѿ" + txStatus + ", Ҫٴ" +
                         txStatus);
            setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null,"Transaction" + txId + " has already " +
                                  txStatus,null,null,null);
        }
                if (sc.getInferiorCount() == 0)
        {
            logger.debug(txType + "" + txId + "ûв,");
            setTxStatus(TxStatus.TX_CONFIRMED);
                                    return TxControl.TX_CONFIRMED;
        }
                 boolean useConfirmOnePhase = TxSetting.getIsUseConfirmOnePhase();
        if (useConfirmOnePhase)
        {
                        if (sc.isAtom())
            {
                if (sc.getInferiorCount() == 1)
                {
                    setTxStatus(TxStatus.TX_CONFIRMING);
                    String infId = (String)sc.getInferiorList().get(0);
                    InferiorResource res =
                        (InferiorResource)sc.getInferiorResource(infId);
                                        MsgObj=res;
                    ObjId=res.getInferiorId();
                                        prepareTime=System.currentTimeMillis();
                    boolean status = res.confirm_one_phase(txId);
                    if (status)
                    {
                        setTxStatus(TxStatus.TX_CONFIRMED);
                        logger.debug(txType + "" + txId + "ִһ׶ύ");
                        confirmTime=System.currentTimeMillis();
                        return TxControl.TX_CONFIRMED;
                    }
                    else
                    {
                        setTxStatus(TxStatus.TX_FAULT);
                        logger.debug(txType + "" + txId + "޷һ׶ύ, ["
                                     + TxUri.getUrlFromUri(infId) +
                                     "]ִconfirm_one_phaseִ");
                        confirmTime=System.currentTimeMillis();
                        return TxControl.TX_FAULT;
                    }
                }
            }
                        if (sc.isCohesion())
            {
                Vector confirmInfs = sc.getConfirmSet();
                if (confirmInfs != null && confirmInfs.size() == 1)
                {
                    setTxStatus(TxStatus.TX_CONFIRMING);
                    String infId = (String)confirmInfs.get(0);
                    InferiorResource res =
                        (InferiorResource)sc.getInferiorResource(infId);
                    String status = res.getSatus();
                    if (status == Status.STATUS_CANCELLED ||
                        status == Status.STATUS_CONFIRMED)
                    {
                        logger.debug("[" + infId +
                                     "]confirm_one_phase, ǰѾ" + status);
                        setTxStatus(TxStatus.TX_FAULT);
                        throw new TxException(null,"inferior)[" +
                                              TxUri.getUrlFromUri(infId) +
                            "]can not confirm_one_phase because has " +
                            status,null,null,null);
                    }
                    boolean statusResult;
                                        if (status == Status.STATUS_PREPARED)
                    {
                        if (!res.confirm(txId))
                        {
                            setTxStatus(TxStatus.TX_FAULT);
                            throw new TxException(null,"inferior(in confirmset)[" +
                                                  TxUri.getUrlFromUri(infId) +
                                                  "]can not confirm",null,null,null);
                        }
                    }
                    else                     {
                        if (!res.confirm_one_phase(txId))
                        {
                            setTxStatus(TxStatus.TX_FAULT);
                            throw new TxException(null,"inferior(in confirmset)[" +
                                                  TxUri.getUrlFromUri(infId) +
                                                  "]can not confirm_one_phase",null,null,null);
                        }
                    }
                    setTxStatus(TxStatus.TX_CONFIRMED);
                                        Vector otherInfs = sc.getNoConfirmSet();
                    for (int i = 0; i < otherInfs.size(); i++)
                    {
                        String inferiorId = (String)otherInfs.get(i);
                        InferiorResource infRes = (InferiorResource)sc.
                                                  getInferiorResource(
                            inferiorId);
                        String infStatus = infRes.getSatus();
                                                if (infStatus != Status.STATUS_CANCELLED &&
                            infStatus != Status.STATUS_CONFIRMED)
                        {
                            boolean cancelResult = infRes.cancel(txId);
                            if (!cancelResult)
                            {
                                                                                            }
                        }
                    }
                    return TxControl.TX_CONFIRMED;
                }
            }
        }
                 String error = null;
        int result = TxControl.TX_FAULT;
                        prepareTime=System.currentTimeMillis();
        int vote = prepare(txId);
                        if (vote == Vote.VOTE_ABORTED)
        {
            cancel(txId);
            result = TxControl.TX_CANCELLED;
            confirmTime=System.currentTimeMillis();
        }         else if (vote == Vote.VOTE_PREPARED)
        {
            result = confirmOnly(txId);
            confirmTime=System.currentTimeMillis();
        }
        else if (vote == Vote.VOTE_RESIGN)
        {
            logger.debug(txType + "" + txId + "ûв,");
            setTxStatus(TxStatus.TX_CONFIRMED);
                                    confirmTime=System.currentTimeMillis();
            return TxControl.TX_CONFIRMED;
        }
        return result;
    }
    
    public int cancel(String txId)
        throws TxException
    {
                          if (sc.getInferiorCount() == 0)
         {
             logger.debug("" + txId + "ûв");
             setTxStatus(TxStatus.TX_CANCELLED);
             return TxControl.TX_CANCELLED;
         }
        String txStatus = sc.getTxStatus();
        String txType = sc.isAtom() ? "ԭ" : "";
                if (txStatus == TxStatus.TX_CONFIRMED ||
            txStatus == TxStatus.TX_CANCELLED)
        {
            logger.debug(txType + "" + txId + "Ѿ" + txStatus + ", Ҫٴȡ");
            setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null,"Transaction" + txId + " has already " +
                                  txStatus,null,null,null);
        }
        boolean atom = sc.isAtom();
        String error = null;
        int result = TxControl.TX_FAULT;
        setTxStatus(TxStatus.TX_CANCELLING);
        logger.debug(txType + "" + txId + "ʼCACNCEL");
                if (true)         {
            Vector infs = sc.getInferiorList();
            for (int i = 0; i < infs.size(); i++)
            {
                String infId = (String)infs.get(i);
                InferiorResource res = (InferiorResource)sc.getInferiorResource(
                    infId);
                String status = res.getSatus();
                                                if (status == Status.STATUS_CANCELLED)
                {
                    logger.debug("[" + infId + "]Ѿ" + status +
                                 ",Ҫٴcancel");
                    continue;
                }
                else if (status == Status.STATUS_CONFIRMED)
                {
                    logger.debug("[" + infId + "]Ѿconfirmed,ܽcancel");
                    error = error + "\n  inferior[" + TxUri.getUrlFromUri(infId) +
                            "] cannot cancel because confirmed" + "\n";
                    continue;
                }
                else
                {   
                                        MsgObj=res;
                    ObjId=res.getInferiorId();
                                        boolean cancelResult = res.cancel(txId);
                    if (!cancelResult)
                    {
                        error = error + "\n  inferior[" +
                                TxUri.getUrlFromUri(infId) +
                                "] cannot cancel" + "\n";
                    }
                                        sc.setInferiorSatuts(infId,res.getSatus());
                                        }
            }
        }
                 if (error == null)
         {
             result = TxControl.TX_CANCELLED;
             setTxStatus(TxStatus.TX_CANCELLED);
                                   }
         else
         {
             setTxStatus(TxStatus.TX_FAULT);
             throw new TxException(null,error,null,null,null);
         }
        return result;
    }
    
    public Hashtable prepareInferiors(String txId, Vector inferiors)
        throws TxException
    {
                 if (sc == null)
         {
             throw new TxException(null,"Transaction:" + txId + " does not exist",null,null,null);
         }
         else
         {
             if (sc.isAtom())
             {
                 setTxStatus(TxStatus.TX_FAULT);
                 throw new TxException(null,"Transaction" + txId +
                     " is atomic, so it cannot prepare part of inferiors.",null,null,null);
             }
         }
        String txStatus = getTxStatus();
        if (txStatus == TxStatus.TX_CONFIRMED ||
            txStatus == TxStatus.TX_CANCELLED)
        {
            logger.debug("" + txId + "Ѿ" + txStatus +
                         ", prepareInferiors");
            setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null,"Transaction" + txId + " has already " +
                                  txStatus,null,null,null);
        }
                 Hashtable result = new Hashtable();
                                   if (inferiors == null)
        {
            inferiors = sc.getInferiorList();
        }
        for (int i = 0; i < inferiors.size(); i++)
        {
            String infId = (String)inferiors.get(i);
            InferiorResource res = (InferiorResource)sc.getInferiorResource(
                infId);
                        if (res == null)
            {
                result.put(infId, Status.STATUS_FAULT);
                logger.debug("ID " + infId + "ڣѾ˳");
            }
            else
            {
                                String status = res.getSatus();
                if (status == Status.STATUS_CANCELLED
                    || status == Status.STATUS_PREPARED
                    || status == Status.STATUS_CONFIRMED
                    || status == Status.STATUS_ABORTED
                    || status == Status.STATUS_RESIGNED)
                {
                    logger.debug("[" + infId + "]Ҫprepare,Ѿ" + status);
                    continue;
                }
                int vote = res.prepare(txId);
                                switch (vote)
                {
                    case Vote.VOTE_ABORTED:
                        result.put(infId, Status.STATUS_ABORTED);
                        break;
                    case Vote.VOTE_PREPARED:
                        result.put(infId, Status.STATUS_PREPARED);
                        break;
                    case Vote.VOTE_RESIGN:
                        result.put(infId, Status.STATUS_RESIGNED);
                        break;
                    case Vote.VOTE_FAULT:
                        result.put(infId, Status.STATUS_FAULT);
                        break;
                    default:
                }
            }
        }
                return result;
    }
    
    public Hashtable cancelInferiors(String txId, Vector inferiors)
        throws TxException
    {
                 if (sc == null)
         {
             throw new TxException(null,"Transaction" + txId + " does not exist",null,null,null);
         }
         else
         {
             if (sc.isAtom())
             {
                 setTxStatus(TxStatus.TX_FAULT);
                 throw new TxException(null,"Transaction:" + txId +
                     " is atomic, so it cannot cancel part of inferiors.",null,null,null);
             }
         }
        String txStatus = sc.getTxStatus();
        if (txStatus == TxStatus.TX_CONFIRMED ||
            txStatus == TxStatus.TX_CANCELLED)
        {
            logger.debug("" + txId + "Ѿ" + txStatus +
                         ", cancelInferiors");
            setTxStatus(TxStatus.TX_FAULT);
            throw new TxException(null,"Transaction" + txId + " has already " +
                                  txStatus,null,null,null);
        }
                 Hashtable result = new Hashtable();
        for (int i = 0; i < inferiors.size(); i++)
        {
            String infId = (String)inferiors.get(i);
            InferiorResource res = (InferiorResource)sc.getInferiorResource(
                infId);
                        if (res == null)
            {
                logger.debug("[" + infId + "]ڣѾ˳");
                result.put(infId, Status.STATUS_FAULT);
            }
            else
            {
                result.put(infId, Status.STATUS_CANCELLING);
                String status = res.getSatus();
                if (status == Status.STATUS_CONFIRMED
                    || status == Status.STATUS_CANCELLED
                    )
                {
                    logger.debug("[" + infId + "]Ѿ " + status + "Ҫcancel");
                }
                else
                {
                    boolean cancelled = res.cancel(txId);
                                        if (cancelled)
                    {
                        result.put(infId, Status.STATUS_CANCELLED);
                    }
                    else                     {
                        result.put(infId, Status.STATUS_FAULT);
                    }
                }
            }
        }
        return result;
    }
    
    public Hashtable requestInferiorStatuses(Vector inferiorIdList)
    {
        Hashtable statuses = new Hashtable();
        for (int i = 0; i < inferiorIdList.size(); i++)
        {
            String infId = (String)inferiorIdList.get(i);
            String status = getInferiorStatus(infId);
            statuses.put(infId, status);
        }
        return statuses;
    }
        public Object GetMsgObject(){
    	return MsgObj;
    }
    public String GetObjId(){
    	return ObjId;
    }
	public long getConfirmTime() {
		return confirmTime;
	}
	public long getPrepareTime() {
		return prepareTime;
	}
	
}
