
package org.act.services.transaction.util;

import java.util.*;
import org.act.services.transaction.util.XName;
public class XElement {
	 
    String xmlheader_ = "<?xml version=\"1.0\" encoding=\"gb2312\"?>\n";
    
    boolean hasheader_ = false;
    
    XName name_;
    
    int index_ = -1;
    
    Hashtable attributes_ = null;
    
    String leafvalue_ = null;
    
    XVector nodes_ = new XVector();
    
    public XElement()
    {
        name_ = new XName("document");
    }
    
    public XElement(String name)
    {
        name_ = new XName(toTagName(name));
    }
    
    public XElement(String name, String value)
    {
        name_ = new XName(toTagName(name));
        leafvalue_ = value;
    }
    
    public void setHeader(boolean h)
    {
        hasheader_ = h;
    }
    
    public String getName()
    {
        return name_.toString();
    }
    
    public String getLocalName()
    {
        return name_.getName();
    }
    
    public String getNamespace()
    {
        return name_.getNamespace();
    }
    
    public void setValue(String value)
    {
        leafvalue_ = value;
    }
    
    public String getValue()
    {
        return leafvalue_;
    }
    
    private String toTagName(String nm)
    {
        int p = nm.indexOf("[");
        if (p >= 0)
        {
            String ret = nm.substring(0, p);
            try
            {
                index_ = Integer.parseInt(nm.substring(p + 1,
                    nm.trim().length() - 1));
            }
            catch (NumberFormatException ne)
            {
                index_ = -1;
            }
            return ret;
        }
        else
        {
            return nm;
        }
    }
    
    public boolean isLeaf()
    {
        return (leafvalue_ != null);
    }
    public int getIndex()
    {
        return index_;
    }
    
    public void addAttribute(String attname, String attval)
    {
        if (attributes_ == null)
        {
            attributes_ = new Hashtable();
        }
        attributes_.put(attname, attval);
    }
    
    public String getAttribute(String attname)
    {
        if (attributes_ == null)
        {
            return null;
        }
        return (String)attributes_.get(attname);
    }
    
    public XElement addElement(XElement e)
    {
        if (e == null)
        {
            return null;
        }
        String nm = e.getName();
        int pos;
        if ((pos = e.getIndex()) <= 0)
        {
            pos = nodes_.getIndex(nm) + 1;             if (pos <= 0)
            {
                pos = 1;
            }
        }
        nm += "[" + pos + "]";
        nodes_.put(nm, e);
        return e;
    }
    
    public XElement getElement(String localname)
    {
        if (localname.endsWith("]"))
        {
            return (XElement)nodes_.get(localname);
        }
        else
        {
            return (XElement)nodes_.get(localname + "[1]");
        }
    }
    
    public int getNodeCount()
    {
        return (nodes_ == null ? 0 : nodes_.size());
    }
    
    public XVector getNodes()
    {
        return nodes_;
    }
    
    public boolean addElement(String path, XElement e, boolean reuse)
    {
        path = normalize(path);
                StringTokenizer st = new StringTokenizer(path, "/");
        XElement prevnode, curnode = this;
        while (st.hasMoreTokens())
        {
            String nm = st.nextToken().trim();
                        prevnode = curnode;
            curnode = curnode.getElement(nm);
            if (curnode == null)
            {
                curnode = prevnode.addElement(new XElement(nm));
            }
            else if (curnode.isLeaf())
            {
                return false;
            }
            else if (!reuse)
            {
                                curnode = prevnode.addElement(new XElement(nm));
            }
        }
                if (curnode != null)
        {
            curnode.addElement(e);
        }
        return true;
    }
    
    public boolean addElement(String elname, String val)
    {
        String path = elname;
        String tagname = path;
        int di = path.lastIndexOf("/");
        if (di >= 0)
        {
            tagname = path.substring(di + 1);
            path = (di == 0 ? "/" : path.substring(0, di));
        }
        else
        {
            path = "/";
        }
                        return addElement(path, new XElement(tagname, val), true);
    }
    
    public String normalize(String path)
    {
        path = path.trim();
        if (path.startsWith("/document"))
        {
            path = path.substring(9).trim();
        }
        if (path.equals("/"))
        {
            return "/";
        }
        StringBuffer np = new StringBuffer();
        if (path.startsWith("/"))
        {
            np.append("/");
        }
        StringTokenizer st = new StringTokenizer(path, "/");
        boolean first = true;
        while (st.hasMoreTokens())
        {
            if (!first)
            {
                np.append("/");
            }
            first = false;
            String elname = st.nextToken().trim();
            np.append(elname);
                    }
        return np.toString();
    }
    
    public String toString()
    {
        
        return (hasheader_ ? xmlheader_ : "") + toString(0);
    }
    
    private String indent(int indent)
    {
        if (indent <= 0)
        {
            return "";
        }
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < indent; i++)
        {
            buf.append(" ");
        }
        return buf.toString();
    }
    
    protected String toString(int indent)
    {
        StringBuffer buf = new StringBuffer(indent(indent)
                                            + "<" + getName());
        if (attributes_ != null)
        {
            Iterator keys = attributes_.keySet().iterator();
            while (keys.hasNext())
            {
                Object key = keys.next();
                buf.append(" " + key.toString() + "=\"" + attributes_.get(key) +
                           "\"");
            }
        }
        buf.append(">");
        if (isLeaf())
        {
            buf.append(leafvalue_ + "</" + getName() + ">");
        }
        else
        {
            for (int i = 0; i < nodes_.size(); i++)
            {
                buf.append("\n" +
                           ((XElement)nodes_.elementAt(i)).toString(indent + 4));
            }
            buf.append("\n" + indent(indent) + "</" + getName() + ">");
        }
        return buf.toString();
    }
    
    public XHashtable toHashtable()
    {
        return toHashtable(false);
    }
    
    public XHashtable toHashtable(boolean local)
    {
        XHashtable h = new XHashtable();
        if (isLeaf())
        {
            h.put("/" + (local ? getLocalName() : getName()), getValue());
        }
        else
        {
            fillHashTable(h, "/" + (local ? getLocalName() : getName()), this,
                          local);
        }
        return h;
    }
    
    private void fillHashTable(XHashtable h, String basename, XElement e,
                               boolean local)
    {
        XVector nodes = e.getNodes();
        for (int i = 0; i < nodes.size(); i++)
        {
            XElement node = (XElement)nodes.elementAt(i);
            if (node.isLeaf())
            {
                h.put(basename + "/" +
                      (local ? node.getLocalName() : node.getName()),
                      node.getValue());
            }
            else
            {
                XVector v = node.getNodes();
                for (int j = 0; v != null && j < v.size(); j++)
                {
                    h.put(basename + "/" +
                          (local ? node.getLocalName() : node.getName()),
                          (XElement)v.elementAt(j));
                }
                fillHashTable(h, basename + "/"
                              + (local ? node.getLocalName() : node.getName()),
                              node,
                              local);
            }
        }
    }
    
    public static void main(String args[])
    {
        XElement book = new XElement("book");
        XElement author = new XElement("author");
        XElement name = new XElement("name", "yr");
        book.addElement(author);
        author.addElement(name);
        System.err.println(book.toString());
                book.addElement("/author", new XElement("", "24"), false);
        book.addElement("author/title", "titleVlaue");
        System.err.println("Ԫ֮");
        System.err.println(book.toString());
        XHashtable hh = book.toHashtable();
        System.err.println(hh);
    }
	
}
