package org.act.services.connector;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;
import org.act.metaset.Metaset;
import org.act.services.ITransportContext;
import org.act.services.ConfigException;
import org.act.services.locale.StringManager;
import org.act.services.pool.IObjectPool;
import org.act.services.protocol.TransportProtocol;
import org.act.services.runtime.IRuntime;
import org.act.services.runtime.lifecycle.LiftcycleException;
import org.act.services.thread.ObjectOperationException;
import org.act.services.thread.Terminatable;
import org.act.services.thread.ThreadPool;
import org.act.services.thread.ThreadPoolRunnable;

public class TransportConnector extends AObjectPoolConnector {
	static Logger log = Logger.getLogger(IRuntime.class.getName());
	static StringManager sm = StringManager.getManager("org.act.services.locale");
	
	private IObjectPool in;							private IObjectPool out;						private ThreadPool threadPool;					private Terminatable trcProcessor;				private List transports = new ArrayList();		
	public TransportConnector(IRuntime runtime) {
		super(runtime);
	}
	
	public void config(Metaset conf) throws ConfigException {
		super.config(conf);
		
				String outputpool = conf.selector(".\\","outputpool");
		in = runtime.getObjectPool(outputpool, IObjectPool.IN_MODE);
		out = runtime.getObjectPool(outputpool, IObjectPool.OUT_MODE);
		
				String tpName = conf.selector(".\\threadpool","name");		threadPool = runtime.getThreadPool(tpName);		
				Metaset protocols = conf.filter(".\\protocol");
		for (int i=0; i<protocols.getChilds().size(); i++) {
			try {
				Metaset protocol = (Metaset) protocols.getChilds().get(i);
				String classname = protocol.selector(".\\","class");
				Class cls = Class.forName(classname);
				if (TransportProtocol.class.isAssignableFrom(cls)) {
					TransportProtocol transportProtocol = (TransportProtocol) cls.newInstance();	
					transportProtocol.setThreadPool(threadPool);
					transportProtocol.setTransportInputPool(in);
					transportProtocol.config(protocol);
					transports.add(transportProtocol);
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ConfigException e) {
				e.printStackTrace();
			}
		}
		
				trcProcessor = new Terminatable() {		
			protected void process() throws ObjectOperationException {
								ITransportContext.ResponseTransportContext obj = null;
				synchronized (out) {
					while (out.empty()) {
						switch (getState()) {
							case RUNNING :
								try {
									out.wait();
								} catch (InterruptedException e) {
									e.printStackTrace();
								}
								break;
							case STOPPING :
								return;
							default :
								throw new IllegalMonitorStateException("");
						}
					}
					obj = (ITransportContext.ResponseTransportContext)out.get();
					out.notifyAll();
				}
								ThreadPoolRunnable comsumer = obj.getResponseHandler();
									threadPool.runIt(comsumer);
			}
		};
	}
	
	
	public void start() {
								for (Iterator iter = transports.iterator(); iter.hasNext();) {
			TransportProtocol element = (TransportProtocol) iter.next();
			try {
				element.start();
			} catch (LiftcycleException e) {
								e.printStackTrace();
			}
		}
				Thread t = new Thread(trcProcessor);
		t.setName("HTTP Transport Output");
		t.start();
	}
	
	
	public void stop() {
				for (Iterator iter = transports.iterator(); iter.hasNext();) {
			TransportProtocol element = (TransportProtocol) iter.next();
			try {
				element.stop();
			} catch (LiftcycleException e) {
								e.printStackTrace();
			}
		}
				trcProcessor.terminate();
		synchronized (out) {
			out.notifyAll();
		}
					}
	
	public Object adapt(Object obj) {
						return null;
	}
	
	
	
}
