
package org.act.services.protocol.http;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

public class ContentLengthInputStream extends FilterInputStream {
    
    
    private int contentLength;
    
    private int pos = 0;
    
    private boolean closed = false;
    
    public ContentLengthInputStream(InputStream in, int contentLength) {
        super(in);
        this.contentLength = contentLength;
    }
    
    public void close() throws IOException {
        if (!closed) {
            try {
                exhaustInputStream(this);
            } finally {
                                                closed = true;
            }
        }
    }
    
    static void exhaustInputStream(InputStream inStream) throws IOException {
                byte buffer[] = new byte[1024];
        while (inStream.read(buffer) >= 0) {
            ;
        }
    }
    
    public int read() throws IOException {
        if (closed) {
            throw new IOException("Attempted read from closed stream.");
        }
        if (pos >= contentLength) {
            return -1;
        }
        pos++;
        return super.read();
    }
    
    public int read (byte[] b, int off, int len) throws java.io.IOException {
        if (closed) {
            throw new IOException("Attempted read from closed stream.");
        }
        if (pos >= contentLength) {
            return -1;
        }
        if (pos + len > contentLength) {
            len = contentLength - pos;
        }
        int count = super.read(b, off, len);
        pos += count;
        return count;
    }
    
    public int read(byte[] b) throws IOException {
        return read(b, 0, b.length);
    }
}
