
package org.act.services.workflow.context;
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;
import org.act.metaset.QName;
import org.act.metaset.Metaset;

public class WorkflowContext {
	private String dictory;
	private Metaset bpel;
		private String name;
	private String targetNamespace;
		private String queryLanguage = "anyURI";
	private String expressionLanguage = "anyURI";
	private String suppressJoinFailure = "yes|no";
	private String enableInstanceCompensation = "yes|no";
	private String abstractProcess = "yes|no";
		private Metaset partnerLinks;
	private Metaset partners;
		private Metaset variables;
	private Metaset correlationSets;
	private Metaset faultHandlers;
	private Metaset compensationHandler;
	private Metaset eventHandlers;
		private Metaset activity;
		private Metaset wsdls;
	private List entryList;
	
	private boolean deploy = true; 
	public WorkflowContext(String dictory) throws Exception {
		this.dictory = dictory;
		this.bpel = getBPEL(this.dictory);
		this.name = this.bpel.selector(".\\", "name");
		this.targetNamespace = this.bpel.selector(".\\", "targetNamespace");
		this.queryLanguage = this.bpel.selector(".\\", "queryLanguage");
		this.expressionLanguage = this.bpel.selector(".\\",
				"expressionLanguage");
		this.suppressJoinFailure = this.bpel.selector(".\\",
				"suppressJoinFailure");
		this.enableInstanceCompensation = this.bpel.selector(".\\",
				"enableInstanceCompensation");
		this.abstractProcess = this.bpel.selector(".\\", "abstractProcess");
		this.partnerLinks = this.bpel.filter("process\\partnerLinks").car();
				if (this.bpel.filter("process\\partners") != null)
			this.partners = this.bpel.filter("process\\partners").car();
		if (this.bpel.filter("process\\variables") != null)
			this.variables = this.bpel.filter("process\\variables").car();
		if (this.bpel.filter("process\\correlationSets") != null)
			this.correlationSets = this.bpel.filter("process\\correlationSets")
					.car();
		if (this.bpel.filter("process\\faultHandlers") != null)
			this.faultHandlers = this.bpel.filter("process\\faultHandlers")
					.car();
		if (this.bpel.filter("process\\compensationHandler") != null)
			this.compensationHandler = this.bpel.filter(
					"process\\compensationHandler").car();
		if (this.bpel.filter("process\\eventHandlers") != null)
			this.eventHandlers = this.bpel.filter("process\\eventHandlers")
					.car();
		this.activity = this.bpel.filter("process\\sequence").car();
		this.wsdls = getWSDLs(dictory);
		this.entryList = new ArrayList();
		setEntryList(this.bpel, this.entryList);
	}
	
	
	private static Metaset getWSDLs(String dirctory) throws Exception {	
		File file = new File(dirctory);
		File[] wsdlFiles = file.listFiles(new FileFilter() {
			public boolean accept(File file) {
				String fn = file.getName();
				int len = fn.length();
				if (len > 5) {
					String ext = fn.substring(len - 5, len);
					return ext.equalsIgnoreCase(".wsdl");
				}
				return false;
			}
		});
		
		Metaset wsdls = new Metaset("wsdls");
		for(int i=0;i<wsdlFiles.length;i++) {
			Metaset wsdl = Metaset.create(wsdlFiles[i].toString());
			wsdls.add(wsdl);
		}
		return wsdls;
	}
	
	
	private static Metaset getBPEL(String dirctory) throws Exception {
		File file = new File(dirctory);
		File[] bpelFiles = file.listFiles(new FileFilter() {
			public boolean accept(File file) {
				String fn = file.getName();
				int len = fn.length();
				if (len > 5) {
					String ext = fn.substring(len - 5, len);
					return ext.equalsIgnoreCase(".bpel");
				}
				return false;
			}
		});
		
		Metaset bpel = Metaset.create(bpelFiles[0].toString());
		return bpel;
	}
	
	private static void setEntryList(Metaset bpel, List entryList) {
		Metaset receives = bpel.filter(".\\receive");
		if (receives != null) {
			for (int j = 0; j < receives.getChilds().size(); j++) {
				Metaset receive = (Metaset) receives.getChilds().get(j);
				if (receive.selector(".\\", "createInstance") != null
						&& receive.selector(".\\", "createInstance").equals(
								"yes")) {
					entryList.add(receive);
				}
			}
		}
		Metaset picks = bpel.filter(".\\pick");
		if (picks != null) {
			for (int j = 0; j < picks.getChilds().size(); j++) {
				Metaset pick = (Metaset) picks.getChilds().get(j);
				if (pick.selector(".\\", "createInstance").equals("yes")) {
					entryList.add(pick);
				}
			}
		}
		for (int i = 0; i < bpel.getChilds().size(); i++) {
			setEntryList((Metaset) bpel.getChilds().get(i), entryList);
		}
	}
	
	private Metaset getWsdlOnNamespace(Metaset wsdl, String Namespace) {
		Metaset wsdls = this.wsdls;
		Metaset Wsdl = (Metaset) wsdls.filter(".\\*",
				"targetNamespace", wsdl.selector(".\\", "xmlns:" + Namespace))
				.car();
		return Wsdl;
	}
	
	public Metaset getWsdlOnNamespace(String Namespace) {
		Metaset wsdls = this.wsdls;
		Metaset wsdl = (Metaset) wsdls.filter(".\\*",
				"targetNamespace",
				this.bpel.selector(".\\", "xmlns:" + Namespace)).car();
		return wsdl;
	}
	
	public Metaset getWsdlOnPartnerLinkType(String partnerLinkType,String partnerRole)
			throws Exception {
		Metaset BPELext_wsdl = Metaset.create(this.dictory + this.name
				+ "_ext" + ".wsdl");
		Metaset pLT = BPELext_wsdl.filter(".\\partnerLinkType", "name",
				partnerLinkType).car();
		Metaset role = pLT.filter(".\\role","name",partnerRole).car();
		String portTypeName = role.selector(".\\portType", "name");
		QName ptName = new QName(portTypeName);
		String ptNameSpace = ptName.getNameSpace();
		return this.getWsdlOnNamespace(BPELext_wsdl, ptNameSpace);
	}
	
	
	public String getWsdl() {
		return this.dictory+this.name+".wsdl";
	}
	
	public String getAbstractProcess() {
		return abstractProcess;
	}
	
	public Metaset getActivity() {
		return activity;
	}
	
	public Metaset getCompensationHandler() {
		return compensationHandler;
	}
	
	public Metaset getCorrelationSets() {
		return correlationSets;
	}
	
	public String getEnableInstanceCompensation() {
		return enableInstanceCompensation;
	}
	
	public List getEntryList() {
		return entryList;
	}
	
	public Metaset getEventHandlers() {
		return eventHandlers;
	}
	
	public String getExpressionLanguage() {
		return expressionLanguage;
	}
	
	public Metaset getFaultHandlers() {
		return faultHandlers;
	}
	
	public String getName() {
		return name;
	}
	
	public Metaset getPartnerLinks() {
		return partnerLinks;
	}
	
	public Metaset getPartners() {
		return partners;
	}
	
	public String getQueryLanguage() {
		return queryLanguage;
	}
	
	public String getSuppressJoinFailure() {
		return suppressJoinFailure;
	}
	
	public String getTargetNamespace() {
		return targetNamespace;
	}
	
	public Metaset getVariables() {
		return variables;
	}
	
	public boolean isDeploy() {
		return deploy;
	}
	
	public void setDeploy(boolean deploy) {
		this.deploy = deploy;
	}
}
