
package org.act.services.workflow.deploy;
import java.io.File;
import java.io.FileFilter;
import java.util.logging.Logger;
import org.act.services.locale.StringManager;
import org.act.services.utils.FileUtil;
import org.act.services.workflow.context.IWorkflowContextPool;
import org.act.services.workflow.context.IWorkflowRuntimeContextPool;
import org.act.services.workflow.context.WorkflowContext;
import org.act.services.workflow.context.WorkflowRuntimeContext;
import org.act.services.workflow.debug.Debug;

public class DeployManager {
	public static final String DEPLOY_DIR = "org.act.services.deployDir";
	public static final String WORKING_DIR = "org.act.services.workingDir";
	static Logger log = Logger.getLogger(DeployManager.class.getName());
	static StringManager sm = StringManager
			.getManager("org.act.services.locale");
	
	public static void deployBPEL() {
		log.info(sm.getString("org.act.wswf.deploy.start"));
		if (Debug.isDebug()) {
			deploy_debug();
		} else {
			deploy_normal("");
		}
		log.info(sm.getString("org.act.wswf.deploy.finish"));
	}
	
	public static void deploy_normal(String name) {
		final String processName = name;
		FileFilter filter = new FileFilter() {
			public boolean accept(File file) {
				String fn = file.getName();
				if (processName.equals("")) {
					int len = fn.length();
					if (len > 4) {
						String ext = fn.substring(len - 4, len);
						return ext.equalsIgnoreCase(".bar");
					}
					return false;
				}
				else{
					return fn.equalsIgnoreCase(processName+".bar");
				}
			}
		};
		String deployDir = System.getProperty(DEPLOY_DIR);
		String workingDir = System.getProperty(WORKING_DIR);
		File workingFiles = new File(workingDir);
		File[] workingBars = workingFiles.listFiles(filter);
		for (int i = 0; i < workingBars.length; i++) {
			File workingBar = new File(workingDir + File.separator
					+ workingBars[i].getName());
			deleteFile(workingBar);
		}
		File deployFiles = new File(deployDir);
		File[] deployBars = deployFiles.listFiles(filter);
		for (int i = 0; i < deployBars.length; i++) {
						String barLoc = workingDir + File.separator;
						File workingFile = new File(barLoc, deployBars[i].getName());
			try {
				FileUtil.inflate(deployBars[i], barLoc
						+ deployBars[i].getName() + File.separator);
				String directory = workingFile.getPath() + File.separator
						+ "meta-inf" + File.separator;
								WorkflowContext wContext = new WorkflowContext(directory);
								IWorkflowContextPool.INSTANCE.put(wContext);
			} catch (Exception e1) {
								e1.printStackTrace();
			}
			log.info(sm.getString("org.act.wswf.deploy.process", deployBars[i]
					.getName()));
		}
	}
	
	private static void deploy_debug() {
		try {
						WorkflowContext wContext;
			wContext = new WorkflowContext(Debug.getDebugPath());
						String name = wContext.getName();
			if (IWorkflowContextPool.INSTANCE.getWorkflowContext(name) != null) {
				IWorkflowContextPool.INSTANCE.remove(name);
			}
			IWorkflowContextPool.INSTANCE.put(wContext);
		} catch (Exception e) {
						e.printStackTrace();
		}
	}
	
	public static boolean anti_deploy(String beplName) {
		for (int i = 0; i < IWorkflowRuntimeContextPool.INSTANCE.size(); i++) {
			WorkflowRuntimeContext rc = (WorkflowRuntimeContext) IWorkflowRuntimeContextPool.INSTANCE
					.get(i);
			if (rc.getWc().getName().equals(beplName)) {
				rc.getWc().setDeploy(false);
				return false;
			}
		}
		IWorkflowContextPool.INSTANCE.remove(beplName);
		String deployDir = System.getProperty(DEPLOY_DIR);
		File deploy = new File(deployDir + File.separator + beplName + ".bar");
		deploy.delete();
		String workingDir = System.getProperty(WORKING_DIR);
		File working = new File(workingDir + File.separator + beplName + ".bar");
		deleteFile(working);
		return true;
	}
	
	private static void deleteFile(File directory) {
		String s[] = directory.list();
		for (int i = 0; i < s.length; i++) {
			File f = new File(directory.getPath() + File.separator + s[i]);
			if (f.isDirectory()) {
				deleteFile(f);
			} else {
				f.delete();
			}
		}
		directory.delete();
	}
}
