
package org.act.services.workflow.runtime;
import java.io.File;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;
import org.act.metaset.Metaset;
import org.act.services.connector.IObjectPoolConnector;
import org.act.services.locale.StringManager;
import org.act.services.pool.IObjectPool;
import org.act.services.runtime.IRuntime;
import org.act.services.runtime.lifecycle.ALifecycle;
import org.act.services.runtime.lifecycle.LiftcycleException;
import org.act.services.thread.ThreadPool;
import org.act.services.workflow.context.IWorkflowContextPool;
import org.act.services.workflow.context.WorkflowContext;
import org.act.services.workflow.record.Records2DB;
import org.act.services.workflow.deploy.DeployManager;

public class WSWFRuntime extends ALifecycle implements IRuntime {
		static Logger log = Logger.getLogger(WSWFRuntime.class.getName());
		static StringManager sm = StringManager
			.getManager("org.act.services.locale");
				private static WSWFRuntime instance = null;
		private Map pairs = new HashMap(7);
		private Map threadpools = new HashMap(5);
		private List connectors = new ArrayList(5);
		public static WSWFRuntime getRuntime() {
		if (instance == null) {
			instance = new WSWFRuntime();
			instance.config();
		}
		return instance;
	}
		public void start() {
		for (Iterator iter = threadpools.values().iterator(); iter.hasNext();) {
			ThreadPool element = (ThreadPool) iter.next();
			element.start();
		}
		for (Iterator iter = connectors.iterator(); iter.hasNext();) {
			IObjectPoolConnector element = (IObjectPoolConnector) iter.next();
			try {
				element.start();
			} catch (LiftcycleException e) {
				e.printStackTrace();
			}
		}
				DeployManager.deployBPEL();
				Records2DB.engine2DB();
				log.info(sm.getString("org.act.wswf.started"));
	}
		public void stop() {
		new Thread() {
			public void run() {
				log.info(sm.getString("org.act.wswf.stop"));
				try {
					Thread.sleep(3000);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
								final int MAX_COUNT = 30;
				int count = 0;
				while (!canStop() && count < MAX_COUNT) {
					try {
						Thread.sleep(200);
						count++;
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				for (Iterator iter = connectors.iterator(); iter.hasNext();) {
					IObjectPoolConnector element = (IObjectPoolConnector) iter
							.next();
					try {
						element.stop();
					} catch (LiftcycleException e) {
						e.printStackTrace();
					}
				}
				for (Iterator iter = threadpools.values().iterator(); iter
						.hasNext();) {
					ThreadPool element = (ThreadPool) iter.next();
					element.shutdown();
				}
				log.info(sm.getString("org.act.wswf.stopped"));
			}
		}.start();	
	}
	
	private boolean canStop() {
		for (Iterator iter = threadpools.values().iterator(); iter.hasNext();) {
			ThreadPool element = (ThreadPool) iter.next();
			if (element.getCurrentThreadsBusy() != 0) {
				return false;
			}
		}
		for (Iterator iter = pairs.values().iterator(); iter.hasNext();) {
			ObjectPoolPair element = (ObjectPoolPair) iter.next();
			synchronized (element.input) {
				if (!element.input.empty()) {
					return false;
				}
			}
			synchronized (element.output) {
				if (!element.output.empty()) {
					return false;
				}
			}
		}
		return true;
	}
		private void config() {
		log.info(sm.getString("org.act.wswf.start"));
		log.info(sm.getString("org.act.wswf.config.start"));
		String config = "conf" + java.io.File.separator + "server.conf";
		String applicationDir = System.getProperty("user.dir");
		try {
			Metaset conf = Metaset.create(config);
			Metaset deploy = conf.filter(".\\Deployer\\DeploySpace");
			if (!deploy.isEmpty()) {
				String deployDir = deploy.car().getValue();
				if (deployDir != null) {
					File dep = null;
					if (deployDir.indexOf(":") >= 0) {
												dep = new File(deployDir);
					} else {
												dep = new File(applicationDir + File.separatorChar
								+ deployDir);
					}
					if (dep.exists() && dep.isDirectory()) {
						System.setProperty(DeployManager.DEPLOY_DIR, dep
								.getCanonicalPath());
						log.info(sm.getString("org.act.wswf.config.deploy", dep
								.getCanonicalPath()));
					} else {
											}
				}
			}
			Metaset working = conf.filter(".\\Deployer\\WorkSpace");
			if (!working.isEmpty()) {
				String workDir = working.car().getValue();
				if (workDir != null) {
					File work = null;
					if (workDir.indexOf(":") >= 0) {
												work = new File(workDir);
					} else {
												work = new File(applicationDir + File.separatorChar
								+ workDir);
					}
					if (work.exists() && work.isDirectory()) {
						System.setProperty(DeployManager.WORKING_DIR, work
								.getCanonicalPath());
						log.info(sm.getString("org.act.wswf.config.working",
								work.getCanonicalPath()));
					} else {
											}
				}
			}
			Metaset pools = conf.filter(".\\objectpools\\objectpool");
			for (int i = 0; i < pools.getChilds().size(); i++) {
				Metaset pool = (Metaset) pools.getChilds().get(i);
				WSWFRuntime.getRuntime().createObjectPool(pool);
			}
			Metaset threadpools = conf.filter(".\\threadpools\\threadpool");
			for (int i = 0; i < threadpools.getChilds().size(); i++) {
				Metaset threadpool = (Metaset) threadpools.getChilds().get(i);
				WSWFRuntime.getRuntime().createThreadPool(threadpool);
			}
			Metaset connectors = conf
					.filter(".\\poolconnectors\\poolconnector");
			for (int i = 0; i < connectors.getChilds().size(); i++) {
				Metaset connector = (Metaset) connectors.getChilds().get(i);
				IObjectPoolConnector conn = WSWFRuntime.getRuntime()
						.createPoolConnector(connector);
				WSWFRuntime.getRuntime().putObjectPoolConnector(conn);
			}
			Metaset protocols = conf
					.filter(".\\poolconnectors\\poolconnector\\protocol");
			for (int i = 0; i < protocols.getChilds().size(); i++) {
				Metaset protocol = (Metaset) protocols.getChilds().get(i);
				Metaset sockets = protocol.filter(".\\socket");
				if (sockets != null) {
					String name = protocol.selector(".\\", "name");
					String version = protocol.selector(".\\", "version");
					String port = sockets.car().selector(".\\", "port");
					String acceptCount = sockets.car().selector(".\\",
							"acceptCount");
					log.info(sm.getString("org.act.wswf.config.protocol", name
							+ version, port, acceptCount));
				}
			}
						Metaset m_addr = conf.filter(".\\DeciderAddress");
			String add = m_addr.car().getValue();
			System.setProperty("DeciderAddress", add);
						String usedb = conf.selector(".\\DataBase", "valid");
			System.setProperty("UseDB", usedb);
			if (usedb.equals("true")) {
				
				log.info(sm.getString("org.act.wswf.config.valid.true"));
								Metaset m_driver = conf.filter(".\\DataBase\\Driver");
				String driver = m_driver.car().getValue();
				System.setProperty("Driver", driver);
				log.info(sm.getString("org.act.wswf.config.database.driver",driver));
								Metaset m_dbaddr = conf.filter(".\\DataBase\\DBaddress");
				String dbadd = m_dbaddr.car().getValue();
				System.setProperty("DBaddress", dbadd);
				log.info(sm.getString("org.act.wswf.config.database.address",dbadd));
								Metaset m_dbuser = conf.filter(".\\DataBase\\DBUser");
				String dbuser = m_dbuser.car().getValue();
				System.setProperty("DBUser", dbuser);
				log.info(sm.getString("org.act.wswf.config.database.user",dbuser));
								Metaset m_dbpw = conf.filter(".\\DataBase\\DBPWD");
				String dbpw = m_dbpw.car().getValue();
				if (dbpw != null) {
					System.setProperty("DBPWD", dbpw);
				} else {
					System.setProperty("DBPWD", "");
				}
			}
			else{
				log.info(sm.getString("org.act.wswf.config.valid.false"));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		log.info(sm.getString("org.act.wswf.config.finish"));
	}
	private class ObjectPoolPair {
		IObjectPool input;
		IObjectPool output;
	}
	public boolean createObjectPool(Metaset config) {
		String name = config.selector(".\\", "name");
		try {
			int size = Integer.parseInt(config.selector(".\\", "size"));
			Class clazz = Class.forName(config.selector(".\\", "class"));
			if (IObjectPool.class.isAssignableFrom(clazz)) {
				ObjectPoolPair pair = new ObjectPoolPair();
				pair.input = (IObjectPool) clazz.newInstance();
				pair.input.setMaxSize(size);
				pair.output = (IObjectPool) clazz.newInstance();
				pair.output.setMaxSize(size);
				pairs.put(name, pair);
				return true;
			}
		} catch (Throwable e) {
			e.printStackTrace();
		}
		return false;
	}
	
	public IObjectPool getObjectPool(String name, int mode) {
		ObjectPoolPair pair = (ObjectPoolPair) pairs.get(name);
		if (pair != null) {
			switch (mode) {
			case IObjectPool.IN_MODE:
				return pair.input;
			case IObjectPool.OUT_MODE:
				return pair.output;
			}
		}
		return null;
	}
	public boolean createThreadPool(Metaset config) {
		String name = config.selector(".\\", "name");
		int minSpareThread = Integer.parseInt(config.selector(".\\",
				"minSpareThread"));
		int maxSpareThread = Integer.parseInt(config.selector(".\\",
				"maxSpareThread"));
		int maxThread = Integer.parseInt(config.selector(".\\", "maxThread"));
		ThreadPool threadPool = new ThreadPool();
		threadPool.setName(name);
		threadPool.setMinSpareThreads(minSpareThread);
		threadPool.setMaxSpareThreads(maxSpareThread);
		threadPool.setMaxThreads(maxThread);
		threadpools.put(name, threadPool);
		return true;
	}
	public ThreadPool getThreadPool(String name) {
		return (ThreadPool) threadpools.get(name);
	}
	public IObjectPoolConnector createPoolConnector(Metaset config) {
		try {
			String cls = config.selector(".\\", "class");
			Class clazz = Class.forName(cls);
			if (IObjectPoolConnector.class.isAssignableFrom(clazz)) {
				Constructor cons = clazz
						.getConstructor(new Class[] { IRuntime.class });
				IObjectPoolConnector connector = (IObjectPoolConnector) cons
						.newInstance(new Object[] { this });
				connector.config(config);
				return connector;
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}
	public void putObjectPoolConnector(IObjectPoolConnector connector) {
		connectors.add(connector);
	}
	public IObjectPoolConnector getObjectPoolConnector(String name) {
		for (Iterator iter = connectors.iterator(); iter.hasNext();) {
			IObjectPoolConnector element = (IObjectPoolConnector) iter.next();
			if (name.equals(element.getName())) {
				return element;
			}
		}
		return null;
	}
	
	public String getWSDLLocation(String serviceName) {
		log.info(sm.getString("org.act.wswf.recWSDL", serviceName));
		WorkflowContext wc = IWorkflowContextPool.INSTANCE
				.getWorkflowContext(serviceName);
		if (wc != null) {
			log.info(sm.getString("org.act.wswf.repWSDL", serviceName));
			return wc.getWsdl();
		}
				return null;
	}
}
