
package org.act.services.wsa;
import org.act.services.soap.model.Body;
import org.act.services.soap.model.Code;
import org.act.services.soap.model.Detail;
import org.act.services.soap.model.Envelope;
import org.act.services.soap.model.FaultCode;
import org.act.services.soap.model.FaultString;
import org.act.services.soap.model.Reason;
import org.act.services.soap.model.SoapAttribute;
import org.act.services.soap.model.SoapQName;
import org.act.services.soap.model.SubCode;
import org.act.services.soap.model.Text;
import org.act.services.soap.model.Value;

public final class WSAFaultUtil {
	
	
	
	public static final String invMessAdProp = "wsa:InvalidMessageAddressingProperty";
	
	
	
	public static final String invAddPropReq = "wsa:MessageAddressingPropertyRequired";
	
	
	
	public static final String invDestUnreach = "wsa:DestinationUnreachable";
	
	
	
	public static final String invActNotSupp = "wsa:ActionNotSupported";
	
	
	
	public static final String invEndponitUnavail = "wsa:EndpointUnavailable";
	
	
	
	public static final void setFaultCode(Envelope env, String soapVersion, String faultType) {
				if ( soapVersion == WSAddressingUtil.S ) {
			setFaultCodeSoap11(env, faultType);
		}
		else {				setFaultCodeSoap12(env, faultType);
		}
	}
	
	
	
	public static final void setFaultCodeSoap11(Envelope env, String faultType) {
		if ( invMessAdProp.equals(faultType) ) {				setFaultCodeSoap11ByType(env, "wsa:InvalidMessageAddressingProperty", "A message addressing property " +
					"is not valid and the message cannot be processed.");
		}
		else if ( invAddPropReq.equals(faultType) ) {				setFaultCodeSoap11ByType(env, "wsa:MessageAddressingPropertyRequired", "A required message addressing" +
					" property is not present.");
		}
		else if ( invDestUnreach.equals(faultType) ) {				setFaultCodeSoap11ByType(env, "wsa:DestinationUnreachable", "No route can be determined " +
					"to reach [destination].");
		}
		else if ( invActNotSupp.equals(faultType) ) {				setFaultCodeSoap11ByType(env, "wsa:ActionNotSupported", "The [action] cannot be processed " +
					"at the receiver.");
		}
		else if ( invEndponitUnavail.equals(faultType) ) {				setFaultCodeSoap11ByType(env, "wsa:EndpointUnavailable", "The endpoint is unable to " +
					"process the message at this time.");
		}
		else {				System.out.println("WSAFaultUtil.setFaultCodeSoap11(): Impossible path, and do nothing");
		}
	}
	
	public static final void setFaultCodeSoap11ByType(Envelope env, String subcode, String reason) {
		WsaFaultSoap11 fault = new WsaFaultSoap11("S11", "Falult");
		FaultCode fc = new FaultCode("faultcode", subcode);
		fault.add(fc);
		FaultString fs = new FaultString("faultstring", reason);
		fs.add(new SoapAttribute(new SoapQName("xml", "lang"), "en"));
		fault.add(fs);
				Body body = (Body)env.getChilds().get(1);
		body.add(fault);
	}
	
	
	
	public static final void setFaultCodeSoap12(Envelope env, String faultType) {
		if ( invMessAdProp.equals(faultType) ) {				setFaultCodeSoap12ByType(env, "S:Sender", "wsa:InvalidMessageAddressingProperty", 
					"A message addressing property is not valid and the message cannot be processed.", 
					"[invalid property]");
		}
		else if ( invAddPropReq.equals(faultType) ) {				setFaultCodeSoap12ByType(env, "S:Sender", "wsa:MessageAddressingPropertyRequired", 
					"A required message addressing property is not present.", "[Missing Property QName]");
		}
		else if ( invDestUnreach.equals(faultType) ) {				setFaultCodeSoap12ByType(env, "S:Sender", "wsa:DestinationUnreachable", "No route can be determined " +
					"to reach [destination].", "empty");
		}
		else if ( invActNotSupp.equals(faultType) ) {				setFaultCodeSoap12ByType(env, "S:Sender", "wsa:ActionNotSupported", "The [action] cannot be processed " +
					"at the receiver.", "[action]");
		}
		else if ( invEndponitUnavail.equals(faultType) ) {				setFaultCodeSoap12ByType(env, "S:Receiver", "wsa:EndpointUnavailable", "The endpoint is unable to " +
					"process the message at this time.", "<wsa:RetryAfter>[xs:NonNegativeInteger]</wsa:RetryAfter>");
		}
		else {				System.out.println("WSAFaultUtil.setFaultCodeSoap11(): Impossible path, and do nothing");
		}
	}
	
	public static final void setFaultCodeSoap12ByType(Envelope env, String code, String subcode, 
			String reason, String detail) {
		
		WsaFaultSoap11 fault = new WsaFaultSoap11("S", "Fault");
		
		Code fcode = new Code("S", "Code");
		Value value = new Value("S", "Value");
		value.setValue(code);
		
		fcode.add(value);
		SubCode scode = new SubCode("S", "Subcode");
		Value valuesub = new Value("S", "Value");
		valuesub.setValue(subcode);
		
		scode.add(value);
		fcode.add(scode);
		fault.add(fcode);
		
		Reason rea = new Reason("S", "Reason");
		Text text = new Text("S", "Text");
		text.add(new SoapAttribute(new SoapQName("xml", "lang"), "en"));
		text.setValue(reason);
		fault.add(rea);
		
		Detail det = new Detail("S", "Detail");
		det.setValue(detail);
		fault.add(det);
		
				Body body = (Body)env.getChilds().get(1);
		body.add(fault);
	}
}
